/* 
 * signal.h 
 */

#ifndef _MY_SIGNAL_H
#define _MY_SIGNAL_H 1

#include <signal.h>

#ifndef SIG_MAX 
#define SIG_MAX 32
#endif

static int sigs[ SIG_MAX ] = {
    SIGHUP,	   /* 1  Hangup (POSIX).  */
    SIGINT,	   /* 2  Interrupt (ANSI).  */
    SIGQUIT,	   /* 3  Quit (POSIX).  */
    SIGILL,	   /* 4  Illegal instruction (ANSI).  */
    SIGTRAP,	   /* 5  Trace trap (POSIX).  */
    SIGABRT,	   /* 6  Abort (ANSI).  */
    SIGIOT,	   /* 7  IOT trap (4.2 BSD).  */
    SIGBUS,	   /* 8  BUS error (4.2 BSD).  */
    SIGFPE,	   /* 9  Floating-point exception (ANSI).  */
    SIGUSR1,	   /* 10 User-defined signal 1 (POSIX).  */
    SIGSEGV,	   /* 11 Segmentation violation (ANSI).  */
    SIGUSR2,	   /* 12 User-defined signal 2 (POSIX).  */
    SIGPIPE,	   /* 13 Broken pipe (POSIX).  */
    SIGALRM,	   /* 14 Alarm clock (POSIX).  */
    SIGTERM,	   /* 15 Termination (ANSI).  */
    SIGSTKFLT,     /* 16 Stack fault.  */
    SIGCLD,	   /* 17 Same as SIGCHLD (System V).  */
    SIGCHLD,	   /* 18 Child status has changed (POSIX).  */
    SIGCONT,	   /* 19 Continue (POSIX).  */
    SIGTSTP,	   /* 20 Keyboard stop (POSIX).  */
    SIGTTIN,	   /* 21 Background read from tty (POSIX).  */
    SIGTTOU,	   /* 22 Background write to tty (POSIX).  */
    SIGURG,	   /* 23 Urgent condition on socket (4.2 BSD).  */
    SIGXCPU,	   /* 24 CPU limit exceeded (4.2 BSD).  */
    SIGXFSZ,	   /* 25 File size limit exceeded (4.2 BSD).  */
    SIGVTALRM,     /* 26 Virtual alarm clock (4.2 BSD).  */
    SIGPROF,	   /* 27 Profiling alarm clock (4.2 BSD).  */
    SIGWINCH,      /* 28 Window size change (4.3 BSD, Sun).  */
    SIGPOLL,	   /* 29 Pollable event occurred (System V).  */
    SIGIO,         /* 30 I/O now possible (4.2 BSD).  */
    SIGPWR,	   /* 31 Power failure restart (System V).  */
    SIGSYS	   /* 32 Bad system call.  */   
}; 

/* we need this typedef for every signal function we use */
typedef void Sigfunc(int);

struct signal_struct {

   int sig;
   Sigfunc *handler;

};

typedef struct signal_struct sig_struct;

Sigfunc *Signal(int, Sigfunc *);
void SetSignalDisp(void *, int);
void ResetSignalDisp(void);
    
#endif
