/* signal.c */

#include <signal.h>
#include "include/signal.h"
#include "include/utility.h"

/*
 * This is the signal funcion rewritten using sigaction
 * The old signal function is not reliable !
 */
Sigfunc *Signal(int signo, Sigfunc *func)
{
   struct sigaction 
     act, 
     oact;
   
   act.sa_handler = func;
   sigemptyset(&act.sa_mask);
   act.sa_flags = 0;
   
   if (signo == SIGALRM) {

#ifdef SA_INTERRUPT
      act.sa_flags |= SA_INTERRUPT; 
#endif
      
#ifdef SA_RESTART
      act.sa_flags |= SA_RESTART; 
#endif      
      
   }

   if (sigaction(signo, &act, &oact) < 0)
     return(SIG_ERR);
   
   return (oact.sa_handler);
}



/*
 *  this function sets the disposition for every given signal
 *  default: set signal to SIG_IGN
 *  the only argument required is an array of structures
 *  this structure has 2 members:
 *  - the signal
 *  - a pointer to the signal handler function
 */
void SetSignalDisp(void *signalsArray, int numSignals)
{
   int
     sigNum;
   
   sig_struct
     *signals;
   
   if (!numSignals) numSignals = SIG_MAX;
   
   /* cast to the right type */
   signals = (sig_struct *) signalsArray;
   
   for (sigNum = 0; sigNum < numSignals; sigNum++) {
      
      if ( (signals+sigNum)->sig < 0)
	break;
      
      if ( Signal( (signals+sigNum)->sig, (signals+sigNum)->handler) == SIG_ERR)
	printMsgExit("signal error");
	
   } /* for */
   
   return;
}


/*
 * this function reset the SIG_IGN for every signal
 */
void ResetSignalDisp(void)
{
   int
     sigNum;
   
   extern int 
     sigs[ SIG_MAX ];
   
   for (sigNum = 0; sigNum < SIG_MAX; sigNum++) {
   
      if ( Signal(sigs[ sigNum  ], SIG_IGN) == SIG_ERR )
	printMsgExit("signal error");
	
   }
   
   return;
}



  
