/* netFuncWrap.c */

#include "include/sockFuncWrap.h"
#include "include/utility.h"

int Socket(int domain, int type, int protocol)
{
   int 
     sockfd;
   
   if ( (sockfd = socket(domain, type, protocol)) < 0 )
     printMsgExit("socket error");
   
   return sockfd;
}


int Bind(int sockfd, struct sockaddr *addr, socklen_t addrlen)
{
   if ( bind(sockfd, addr, addrlen) < 0 )
     printMsgExit("bind error");
   
   return 0;
}

int Listen(int  sockfd, int backlog)
{
   if ( listen(sockfd, backlog) < 0 )
     printMsgExit("listen error");
   
   return 0;
}

/*
 * this version of Accept takes care of the interrupted system calls
 * SCENARIO:
 * the parent process forks some children and then blocks in a 
 * "accept" slow call (slow = could never return)
 * eventually a child is killed and the parent process which gets SIGCHLD
 * process the signal and jumps to the signal handler.
 * when the signal handler returns "accept" returns EINVAL since the system
 * call was interrupted.
 * we must handle this error restarting the system call
 */
int Accept(int sockfd, struct sockaddr *addr, socklen_t *addrlen)
{
   int 
     connfd;

   for ( ; ; ) {
      
      if ( (connfd = accept(sockfd, addr, addrlen)) < 0 ) {
	 
	 if (errno == EINVAL) {
	    
	    printf("\naccept error: got EINVAL\n");
	    continue;
	    
	 } else {
	    
	    printMsgExit("accept error");
	 }
	 
      }

      return connfd;
 
   } /* for */
   
}

int Connect(int sockfd, struct sockaddr *addr, socklen_t addrlen)
{
   int 
     connfd;
   
   if ( (connfd = connect(sockfd, addr, addrlen)) < 0 )
     printMsgExit("connect error");
   
   return 0;
}

int Inet_pton(int domain, const char *str_addr, void *addrptr)
{
   if (inet_pton(domain, str_addr, addrptr) < 0 )
     printMsgExit("inet_pton error");
   
   return 0;
}


int sockClose(int sockfd)
{
   if ( close(sockfd) < 0 ) {
     
      printf("close error: pid = %d, ppid = %d\n", (int) getpid(), (int) getppid());
      printf("sockfd = %d\n", sockfd);
      exit(0);
   }
   
   return 0;
}
   

int Sendto(int sockfd, const void *mesg, size_t len, int flags, const struct sockaddr *to, socklen_t tolen)
{
   int bytes_sent;
   
   if ( (bytes_sent = sendto(sockfd, mesg, len, flags, to, tolen)) < 0) {
      
      fprintf(stderr, "errno = %d\n", errno); 
      perror("sendto error");
      
      printMsgExit("sento error");
    
   }

   return bytes_sent;
}


int Recvfrom(int sockfd, void *buf, size_t len, int flags, struct sockaddr *from, socklen_t *fromlen)
{
   int bytes_received;
   
   if ( (bytes_received = recvfrom(sockfd, buf, len, flags, from, fromlen)) < 0) {


      perror("recvfrom error");	 
      printf("errno = %d\n", errno); 
      printMsgExit("recvfrom error");
   
   }

   return bytes_received;
}

