/*
 * udpClient.c
 */
#include "include/sockFuncWrap.h"
#include "include/unp.h"
#include "include/fork.h"
#include "include/signal.h"
#include "include/utility.h"
#include <fcntl.h>

#define LISTEN_PORT 6000
#define SEND_PORT 5000
#define MAX_HOST_LENGTH 64
#define TMP_FILE "/tmp/tmp.log"


void do_cmd(int, struct sockaddr_in*, int, char *);
void create_listen_socket(int *, struct sockaddr_in *, int);

int main(int argc, char **argv)
{
   int listen_sockfd;
   int listen_port;
   int send_port;
   char *send_host;
   struct sockaddr_in servaddr;

   /*
    * reset signal disposition
    */
   ResetSignalDisp();
   send_host = (char *) malloc(MAX_HOST_LENGTH);
   fprintf(stderr, "usage: %s <host> [ send port ] [ listen port ]\n", argv[ 0 ]);
   fprintf(stderr, "use: quit to exit\n");
   if (argc == 4) {

      send_port   = atoi(argv[ 2 ]);      
      listen_port = atoi(argv[ 3 ]);
      printf("now listening on port %d\n", listen_port);
      
   } else if (argc == 3) { 

      send_port   = atoi(argv[ 2 ]);      
      listen_port = LISTEN_PORT;
      
   } else if (argc == 1) {
      printf("paramenter host required:\n");
      exit(0);      
   } else {
      
      listen_port = LISTEN_PORT;
      send_port = SEND_PORT;
      printf("no port specified\n");
      printf("now listening on default port %d\n", listen_port);
      
   }  
   strncpy(send_host, argv[ 1 ], strlen(argv[ 1 ]));
   
   create_listen_socket(&listen_sockfd, &servaddr, listen_port);   
   do_cmd(listen_sockfd, &servaddr, send_port, send_host);
   
   fprintf(stderr, "I quit!\n");   
   exit(0);
}

/*
 * do_cmd
 */
void do_cmd(int listen_sockfd, struct sockaddr_in *servaddr, int send_port, char *send_host)
{
   int send_sockfd;
   int bytes_in;
   int ready = 0;
   char *cmd;
   char *buff;
   struct sockaddr_in cliaddr;
   socklen_t from_len;
   size_t size = MAXLINE;
   fd_set readfds[1];
   struct timeval tv;
   

   tv.tv_sec = 1;
   tv.tv_usec = 0;
   FD_ZERO(readfds);
   FD_SET(listen_sockfd, readfds);
   cmd  = (char *) malloc(MAXLINE);
   buff = (char *) malloc(MAXLINE);   
   /*
    * create send socket
    */
   bzero(&cliaddr, sizeof(cliaddr));
   cliaddr.sin_family = AF_INET;
   cliaddr.sin_port = htons(send_port);
   Inet_pton(AF_INET, send_host, (void *) &cliaddr.sin_addr);
   send_sockfd = Socket(AF_INET, SOCK_DGRAM, 0);
   setsockopt(send_sockfd, SOL_SOCKET, SO_SNDBUF, &size, (socklen_t) sizeof(size));
         
   for ( ; ; ) {
      
      memset(buff, 0, MAXLINE);
      memset(cmd, 0, MAXLINE);
      FD_ZERO(readfds);
      FD_SET(listen_sockfd, readfds);
      
      fprintf(stderr, "\n$ ");
      fgets(cmd, MAXLINE, stdin);
      fflush(NULL);
      if (!strncmp(cmd, "quit", 4)) {
	 Sendto(send_sockfd, cmd, strlen(cmd), 0, (SA *) &cliaddr, sizeof(cliaddr));	 
	 close(send_sockfd);
	 close(listen_sockfd);
	 return;
      }
      strncat(cmd, "\0", 1);
      Sendto(send_sockfd, cmd, strlen(cmd), 0, (SA *) &cliaddr, sizeof(cliaddr));
      
      ready = select(listen_sockfd + 1, readfds, NULL, NULL, &tv);

      if (ready != 0) {	 
	 bytes_in = Recvfrom(listen_sockfd, (void *) buff, MAXLINE, 0, (SA *) servaddr, &from_len);
      }
      
      if (!strncmp(buff, "quitting", 8)) {
	 fprintf(stderr, "quitting\n");
	 close(send_sockfd);
	 close(listen_sockfd);
	 return;
      }
      
      fprintf(stderr, "%s\n", buff);
      fflush(NULL);
   }
   
   return;
}


/*
 * create a udp listen socket
 */
void create_listen_socket(int *sockfd, struct sockaddr_in *servaddr, int listen_port)
{
   /*
    * create the socket
    */
   *sockfd = Socket(AF_INET, SOCK_DGRAM, 0);
   fcntl(*sockfd, F_SETFL, O_NONBLOCK);
   
   /*
    * clean up servaddr
    */
   bzero(servaddr, sizeof(struct sockaddr_in));
   
   /*
    * set socket parameters
    */
   servaddr->sin_family = AF_INET;
   servaddr->sin_addr.s_addr = htonl(INADDR_ANY);
   servaddr->sin_port = htons(listen_port);
   
   /*
    * Bind the socket
    */
   Bind(*sockfd, (SA *) servaddr, sizeof(struct sockaddr_in));

   return;
}
